use {Message, Channel, User};

use std::sync::Arc;

#[derive(Debug)]
pub enum Event {
    Message { message: Message },     // A user sends a message

    SelfJoin { channel: Channel },    // We join a channel
    SelfQuit { channel: Channel },    // We quit a channel

    UserJoin { channel: Channel, user: User }, // A user joins a channel
    UserQuit { channel: Channel, user: User }  // A user quits a channel
}

#[derive(Debug)]
pub struct Envelope {
    pub from: Option<String>,
    pub event: Event,
    pub to: Vec<String>,
    pub tags: Vec<String>
}

impl Envelope {
    pub fn new (event: Event) -> Envelope {
        let mut tags: Vec<String> = vec![];

        match &event {
            &Event::Message { ref message } => {
                tags.push(format!("user:{:}", message.author.name));
                match message.channel {
                    Some(ref channel) => {
                        tags.push(format!("channel:{:}", channel.name));
                    },
                    None => {}
                }
            },
            &Event::SelfJoin { ref channel } => {
                tags.push(format!("channel:{:}", channel.name));
            },
            &Event::SelfQuit { ref channel } => {
                tags.push(format!("channel:{:}", channel.name));
            },
            &Event::UserJoin { ref channel, ref user } => {
                tags.push(format!("channel:{:}", channel.name));
                tags.push(format!("user:{:}", user.name));
            },
            &Event::UserQuit { ref channel, ref user } => {
                tags.push(format!("channel:{:}", channel.name));
                tags.push(format!("user:{:}", user.name));
            }
        }

        Envelope {
            from: None,
            event: event,
            to: vec![],
            tags: tags
        }
    }
}
